#functions needed to replicate analyses in Miles and Vaisey 2015, "Morality and Politics: Comparing Alternate Theories"

#compare.surveys() examines the linear relationships among all variables and determines whether they differ by the survey they come from
#x is a named list of variables 
#returns pairs of variables whose relationship differs across data sets
compare.surveys=function(x, bicdiff=2){
  combos=combn(x, 2, simplify=F)
  diffmat=matrix(NA, ncol=2)
  for(i in 1:length(combos)){
    tempdat=data.frame(yvar=combos[[i]][[1]], xvar=combos[[i]][[2]], mmsurvey=d3$mmsurvey, wgt=d3$weight)
    tempdat=na.omit(tempdat)
    m1=lm(yvar ~ xvar, data=tempdat, weights=wgt)
    m2=lm(yvar ~ xvar*mmsurvey, data=tempdat, weights=wgt)
    fitstats=bic(m1, m2)
    isdiff=(fitstats["m2","BIC"]-fitstats["m1","BIC"]) < -1*bicdiff
    if(isdiff) diffmat=rbind(diffmat, names(combos[[i]]))
  } #end i loop
  return(diffmat)
} #end compare.surveys()

#this is a modified form of a function that Miles has written for other purposes that gives a variety of fit statistics - only some of the functionality is used in this paper
bic = function (..., c=TRUE, digits=1) {
	mods = list(...)
	nmods = length(mods)
	modnames = as.character(match.call())[2:(nmods+1)]
	bic = c(); aic = c(); obs = c(); classes=c(); family=c(); link=c(); aicc=c(); raftbic=c(); ibic=c()
	for(i in 1:nmods){
		mod=mods[i][[1]]
		modtype=class(mods[i][[1]])[1]
		#for glm's and lm's
		if(modtype %in% c("glm", "lm", "negbin")){
			n = length(mod$model[,1])
			family=c(family, switch(modtype, family(mod)$family, negbin="neg. binomial"))
			link=c(link, family(mod)$link)
			infomat=solve(vcov(mod))
		}
		#calculate fit statistics
		idet <- det(infomat)
		ll = logLik(mod)
		p = switch(modtype, attr(ll, "df"), vglm=(slot(mod, "rank")+1) )
		bic = c(bic, -2*ll + log(n)*p)
		ibic = c(ibic, -2*ll + p*log(n/2*pi) + log(idet))
		aic = c(aic, -2*ll + 2*p)
		aicc = c(aicc, -2*ll + 2*p + (2*p*(p+1))/(n-p-1))
		obs = c(obs, n)
		classes = c(classes, modtype)
	} #end of if statement
	#round results
	bic=round(bic, digits=digits)
	ibic=round(ibic, digits=digits)
	aic=round(aic, digits=digits)
	aicc=round(aicc, digits=digits)
	if (c) df = data.frame(class=classes, family=family, link=link, BIC=bic, IBIC=ibic, AIC=aic, AIC.c=aicc, N=obs, row.names=modnames) else 
	df = data.frame(class=classes, family=family, link=link, BIC=bic, IBIC=ibic, AIC=aic, N=obs, row.names=modnames)
	return(df)
}

#this gets weighted descriptive statistics
get.desc=function(x, vars){
  x=x[,vars]
  means=apply(x, 2, wtd.mean, weights=x$weight)
  sds=apply(x, 2, function(y, ...) ifelse(range(y, na.rm=T)[2]==1 & range(y, na.rm=T)[1]==0, NA, sqrt(wtd.var(y, na.rm=T, weights=x$weight))))
  return(cbind(means, sds))
}

#combines multiply imputed data without calculating a standard error
#take a list of imputation results
mi.pointest=function(x){
	mimat=x[[1]]
	for(i in 2:length(x)){
		mimat=mimat + x[[i]]
	}
	mimat/length(x)
}

#function to perform factor analysis with MI data
mi.fa=function(dat, whichvars=sort.names, nfacts=3){
  outdat=dat[,whichvars]
	famod=fa(outdat, nfactors=nfacts, score="tenBerge", rotate="promax")
}

#function to identify the columns from a factor analysis and name them consistently
rename.factors=function(x){
  loads=loadings(x)
  isorder=loads["moralrelig",] > .30 & loads["sv.conform",] > .3 & loads["sv.trad",] > .3
  isself=loads["sv.hedon",] > .3 & loads["sv.power",] > .3 & loads["sv.stim",] > .3
  isother=loads["sv.univ",] > .2 & loads["mfq.fair",] > .3 & loads["mfq.harm",] > .3
  colnames(loads)[isorder]=colnames(x$scores)[isorder]="order"
  colnames(loads)[isself]=colnames(x$scores)[isself]="self.focus"
  colnames(loads)[isother]=colnames(x$scores)[isother]="other.focus"
  x$loadings = loads
  return(x)
}

#function to extract the correlation of factors and factor scores from a factor analysis object
extract.cors=function(x){
  factnames=colnames(loadings(x))
  cors=sqrt(x$R2)
  names(cors)=factnames
  cors[sort(names(cors))]
}

#function to standardize estimates from a MI model
mi.z=function(mods, std.coef="beta", white.adjust="HC3"){
  z.coef=list(coef(mod.summary(mods[[1]], std.coef=std.coef, verbose=F))[,1])
  z.var = list(vcovHC(mods[[1]], type=white.adjust))
  for(i in 2:length(mods)){
    z.coef[[i]]=coef(mod.summary(mods[[i]], std.coef=std.coef, verbose=F))[,1]
    z.var[[i]]=vcovHC(mods[[i]], type=white.adjust)
  }
  outmod=MIcombine(z.coef, z.var)
  return(outmod)
}

#  this is a multi-purpose function that Miles has written (and continually tinkers with) - most of the functionality is not used in this paper
mod.summary = function(model, white.adjust=c("none", "const", "HC3", "HC0", "HC1", "HC2", "HC4", "cluster"), cluster=NULL, std.coef=c("none", "beta", "y", "mix", "robust"), bic=TRUE, odds=FALSE, CI=FALSE, CIlevel=0.95, logit.tools=T, verbose=T, ...) {

#function to estimate the variance of the latent variable underlying a binary or ordinal logit outcome
#note: results differ slightly from listcoef command in fitstat, and a little more when there are weights (but I think it is because listcoef converts pweights to aweights)
estyvar=function(m, ...) {
		#adapted from fitstat in Stata
		#modtype=class(m)[1]
		#if(modtype=="glm") modtype=family(m)$link
		mod.matrix=switch(modtype, ologit=model.matrix(m$terms, data=m$model), oprobit=model.matrix(m$terms, data=m$model), ologit.clm=model.matrix(m$terms, data=m$model), model.matrix(m$formula, data=m$model))
		#are there any weights?
		if("(weights)" %in% colnames(m$model)){
			w = m$model[,"(weights)"]
		} else
		w = 1
		#this makes all data deviated from its mean
		mod.matrix=apply(mod.matrix,2,function(x) {x-mean(x)})
		new.n=dim(mod.matrix)[1]-1
		#get the X matrix without the intercept
		mod.matrix=mod.matrix[,colnames(mod.matrix)!="(Intercept)"]
		require(car)
		#note the crossprod is just giving you t(X) %*% X
		Vx = wcrossprod(mod.matrix, mod.matrix, w)/new.n
		#adjust for single variable models where the matrix becomes a vector
		if(class(Vx)=="numeric") Vx = matrix(Vx, nrow=1) 
		#explained variance (variance of the linear predictor), make sure there is no intercept
		b=switch(modtype, ologit.clm=m$beta, coef(m))
		b=b[names(b)!="(Intercept)"]
		#t(b) gives a row vector, and t(t(b)) gives a column vector
		ss.ex=t(b) %*% Vx %*% t(t(b))
		#error variance, assumed to be 3.29 in logit models, 1 in probit
		sse = switch(modtype, ologit=pi^2/3, ologit.clm=pi^2/3, logit=pi^2/3, probit=1, oprobit=1)
		totalvar=ss.ex + sse
		if(verbose) print(paste("SD of latent variable", round(sqrt(as.numeric(totalvar)), 4), sep=": "))
		return(as.numeric(totalvar))
}

#standardized, y-standardized, and robust regression coefficients - assumes your model has an intercept
coeffs=function(MOD, type, mod.sumry, ...)
	{
		#get the model matrix (this handles any categorical variable correctly)
    dat=as.data.frame(model.matrix(MOD$terms, data=MOD$model))
    b = switch(modtype, lm=summary(MOD)$coef[-1,1], logit=summary(MOD)$coef[-1,1], ologit=coef(MOD), oprobit=coef(MOD), probit=coef(MOD)[-1], ologit.clm=MOD$beta)
		se = switch(modtype, lm=sqrt(diag(mod.sumry$vcov))[-1], logit=sqrt(diag(mod.sumry$vcov))[-1], probit=sqrt(diag(mod.sumry$vcov))[-1], ologit=sqrt(diag(vcov(mod.sumry)))[1:length(coef(MOD))], oprobit=sqrt(diag(vcov(mod.sumry)))[1:length(coef(MOD))], ologit.clm=sqrt(diag(mod.sumry$vcov))[(length(MOD$alpha)+1):length(coef(MOD))])
		if(type!="y") sx = apply(dat[-1], 2, sd)
		sy = switch(modtype, lm=apply(MOD$model[1], 2, sd), logit=sqrt(estyvar(MOD, ...)), probit=sqrt(estyvar(MOD, ...)), ologit=sqrt(estyvar(MOD, ...)), oprobit=sqrt(estyvar(MOD, ...)), ologit.clm=sqrt(estyvar(MOD, ...)))
		if(type=="beta") {beta = b * sx/sy; ses=se*(sx/sy)}
		if(type=="y") {beta = b/sy; ses=se/sy}
    		if(type=="mix") {
	      get.uniques=sapply(dat[-1], unique, simplify=FALSE)
	      lengths=sapply(get.uniques, length)
	      max1=sapply(get.uniques, max)
	      min0=sapply(get.uniques, min)
	      is.binary=lengths==2 & max1==1 & min0==0
	      beta = ifelse(is.binary, b/sy, b * sx/sy)
	      ses = ifelse(is.binary, se/sy, se*(sx/sy))
    		}
		if(type=="robust"){
			sx <- sapply(MOD$model[-1], IQR)
    			sy <- sapply(MOD$model[1], IQR)
    			beta <- b * sx/sy
    			ses=se*(sx/sy)
		}
		return(cbind(beta, ses))
	}

#Confidence intervals - I have to include this because the built in function confint() won't use corrected vcov matrices
confints=function(object, parm, level = 0.95, ...)
	{
    	cf <- object$coef[,1]
    	pnames <- names(cf)
    	if (missing(parm)) 
        	parm <- pnames
    	a <- (1 - level)/2
    	a <- c(a, 1 - a)
    	pct = paste(round(100 * a, 1), "%")
      	fac <- qt(a, object$df[2])
    	ci <- array(NA, dim = c(length(parm), 2L), dimnames = list(parm, pct))
    	ses <- sqrt(diag(object$vcov))[parm]
    	ci[] <- cf[parm] + ses %o% fac
    	ci
	}

#Corrected vcov matrices based on clustering (rather than just assuming a generic approach like HC0 or HC3)
clx = function(fm, dfcw, cluster){
         # R-codes (www.r-project.org) for computing
         # clustered-standard errors. Mahmood Arai, Jan 26, 2008.
	 # The arguments of the function are:
         # fitted model, degree of freedom correction (not used in my implementation) and cluster) 
         # reweighting the var-cov matrix for the within model (i.e. the meat part of the sandwhich matrix)
         #first make sure you have the same observations in your cluster variable that were used in the fitted model
         if(length(cluster) != length(unlist(labels(fm$model)[1]))) cluster.mod = cluster[as.numeric(unlist(labels(fm$model)[1]))]
         else cluster.mod=cluster
         M <- length(unique(cluster.mod))   
         N <- length(cluster.mod)          
         K <- fm$rank                        
         dfc <- (M/(M-1))*((N-1)/(N-K))  #finite sample adjustment
         uj  <- apply(estfun(fm),2, function(x) tapply(x, cluster.mod, sum))  #i.e. summing within each unique level of cluster.mod for each row in the matrix containing the empirical estimating functions of the fitted model
         #NOTE: the crossprod() is equivalent to sum(u*t(u)), i.e. the sum of the vector created by u*transpose(u)
         vcovCL <- dfc*sandwich(fm, meat=crossprod(uj)/N)*dfcw
	}

#Warnings and data checks	
	if(odds) {
		if (class(model)[1] != "glm") stop("Fitted model must of class 'glm' in order to use odds ratios")
		if (model$family$link != "logit") stop("Cannot produce odds ratios, model is not a logistic regression")
	}
	if(!require(sandwich)) stop("Required sandwich package is missing")
	if(!require(lmtest)) stop("Required lmtest package is missing")
	if(match.arg(white.adjust)=="cluster") if(is.null(cluster)) stop("Must specify a variable representing hetero. clustering to use the 'cluster' option in white.adjust")

#type of model
modtype=class(model)[1]
if(modtype=="glm") modtype=model$family$link
if(modtype=="polr") modtype=switch(model$method, logistic="ologit", probit="oprobit")
if(modtype=="sclm") {modtype="ologit.clm"; model$rank=model$edf}

#Calculating vcov matrix for SEs
	adjust=match.arg(white.adjust)
	if(adjust=="none") V = vcov(model)
	if(adjust=="cluster") V = clx(model, 1, cluster)
	if((adjust != "none") & (adjust != "cluster")) V = vcovHC(model, type=adjust)
	
#Building the summary object
	sumry = summary(model)
	table = coeftest(model, vcov=V)
	sumry$cov.scaled=sumry$vcov=V

#drop weights from model matrix if there are any - allows the standardization functions to run as they are currently written
if("(weights)" %in% names(model$model)) {
	model$model = model$model[-which(names(model$model)=="(weights)")]
}

#Update coefficients based on specified options	
	if(CI) {
#if you can just change the relevant features of a model object, you can use the built in functions confint() and not have to rewrite them - i.e. you need to change the vcov matrix that the confint() functions get
		table2 = confints(sumry, level=CIlevel)
		if(odds) table2=exp(table2)
		if (match.arg(std.coef)!="none")
		{
			cimod=model
			for(i in 1:2)
			{
				cimod$coefficients=table2[,i]
				table2[,i]=c(NA,coeffs(cimod, std.coef)[,1])
			}
		} 
		sumry$confint=table2
	} #end if statement for confident intervals
	#modify this so that it uses corrected SEs
	if (match.arg(std.coef)!="none") {
		if(modtype %in% c("lm", "logit", "probit")) {
      		table[,c(1,2)]= rbind(rep(NA, 2), coeffs(model, std.coef, sumry, ...))
			table[1,] = rep(NA, 4)
			}
		if(modtype %in% c("ologit", "oprobit")) table[,c(1,2)]= coeffs(model, std.coef, sumry, ...)
		if(modtype == "ologit.clm") {
			table[(length(model$alpha)+1):length(coef(model)), c(1,2)] = coeffs(model, std.coef, sumry, ...)
			table[1:length(model$alpha),]=NA
			}
		nms=colnames(table)
		nms[1]=switch(std.coef, beta="Std. coef", y="y-std. coef", mix="Mixed y/std. coef", robust="Robust coef")
		colnames(table)=nms
	} #end if statement for std.coef
	if(odds) {
		nms=colnames(table)
		nms=c(nms[1], "Odds ratio", nms[2:4])
		table=cbind(table[,1], exp(coef(model)), table[,2:4])		
		colnames(table)=nms
	} #end if statement for odds	
	sumry$coefficients = table
#additional information to attach to summary object
	sumry$bic = AIC(model, k=log(length(model$fitted.values)))
#add extra fit information for logit regressions
if(modtype=="logit" & logit.tools==T){
	#requires my function logit.tools
	source('~/Documents/R code/logit.tools.R', chdir = TRUE)
	sumry$logit.fit = logit.tools(model, verbose=F, ...)
}

#information for printing
if(verbose){
	cat("Information about displayed results", "\n")
	if(bic) {
		cat("BIC: ", sumry$bic, "\n")
	}
	if(odds) cat("Odds ratios used", "\n")
	if((adjust != "const") & (adjust != "none"))  cat("Note: Heteroscedasticity-consistent SEs using adjustment", adjust, "\n")
	if(match.arg(std.coef)=="beta") cat("Note: Standardized regression coefficients/SEs used", "\n")
	if(match.arg(std.coef)=="robust") cat("Note: Robust standardized regression coefficients/SEs used", "\n")
	if(match.arg(std.coef)=="y") cat("Note: y-standardized regression coefficients/SEs used", "\n")
  if(match.arg(std.coef)=="mix") cat("Note: Standardized regression coefficients/SEs used for non-binary predictors", "\n", "y-standardized regression coefficients/SEs used for binary predictors", "\n")
	if(CI) {
		cat("\n")
		cat(CIlevel*100,"% confidence intervals", "\n")
		print(table2)
		}
} #end if(verbose) statement
			return(sumry)
	}
	
	
